//
//  TweetsTableViewController.swift
//  Busca Pios
//
//  Created by Santiago Pavón on 15/12/14.
//  Copyright (c) 2014 UPM. All rights reserved.
//

import UIKit

private let TWITTER_SEARCH_URL = "https://api.twitter.com/1.1/search/tweets.json"


class TweetsTableViewController: UITableViewController {

    // String a buscar en Twitter
    var query: String?
    
    // Los tweets encontrados tras la busqueda.
    // Es un array de objetos Tweet.
    var tweets = [Tweet]()
    
    // Almacen de profile images.
    // La clave es un String con el URL, y el valor es un objeto UIImage.
    var images = [String:UIImage]()
    
    // URL Session para descargar tweets
    var session: URLSession!
    
    // Estado del boton para bajar mas tweets.
    var downloadButtonEnabled: Bool = false {
        didSet {
            if let cell = tableView.cellForRow(at: IndexPath(row: 0, section: 1)) as? NextPageCell {
                cell.downloadButton.isEnabled = downloadButtonEnabled
            }
        }
    }
    
    // Query para bajar la siguiente pagina de Tweets.
    var current_query: String?
    
    override func viewDidLoad() {
        super.viewDidLoad()
                
        //---
     
        tableView.rowHeight = UITableViewAutomaticDimension
        tableView.estimatedRowHeight = 110
        
        //---
        
        autenticaYDescarga()
    }

    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        
        images = [:]  // Elimina las imagenes guardadas.
    }

    
    private func autenticaYDescarga() {
        self.title = "Autenticando ..."
        incrNetActivity()
        
        OAuthToken.getAccessToken() {(access_token: String?) in
            
            DispatchQueue.main.async {
                
                self.decrNetActivity()
                
                if access_token != nil {
                    self.title = "Autenticado"
                    
                    self.createSession(access_token: access_token!)
                    self.getTweets()
                } else {
                    // No pudo realizarse la conexion o la descarga fallo.
                    self.title = "Error autenticando";
                }
            }
        }
    }
    
    // MARK: - URL Session para bajar los Tweets

    
    private func createSession(access_token: String) {
        
        print("Creando sesion para bajar tweets.")
        
        let config = URLSessionConfiguration.default
        
        // Añadimos la cabecera de autenticacion necesaria para acceder al API de busquedas de Twitter.
        config.httpAdditionalHeaders = ["Authorization": "Bearer \(access_token)"]
     
        session = URLSession(configuration: config)
    }
    
    
    // MARK: - Search Tweets
    

    
    /**
    *  Obtiene todos los tweet que encajan con el string buscado (self.query).
    *
    *  Los tweets encontrados se dividen en paginas que hay que descargarse una a una.
    *
    *  Los tweets encontrados se almacenan en self.tweets.
    */
    private func getTweets() {
        
        // Construye la primera query que hay que añadir a la URL.
        // Con esta query solo se baja la primera pagina de tweets.
        current_query = "?q=\(self.query!)&count=100".addingPercentEncoding(withAllowedCharacters: .urlQueryAllowed)
    
        downloadNextPage()
    }
    
    // Descarga los tweets indicado por current_query.
    func downloadNextPage() {
        
        self.title = "Downloading ..."
        incrNetActivity()
        downloadButtonEnabled = false
        
        // Construimos la URL añadiendo la query current_query.
        
        let pageURL = "\(TWITTER_SEARCH_URL)\(current_query!)"
        
        print("Page URL = \(pageURL)")
        
        if let url = URL(string: pageURL) {
            
            let task = session.dataTask(with: url) { (data: Data?, res: URLResponse?, error: Error?) in
                
                let code = (res as! HTTPURLResponse).statusCode
                print("code = \(code) ", terminator: "")
                if error == nil && code == 200 {
                    // Procesamos el JSON recibido.
                    self.addJSONData(jsonData: data!)
                } else {
                    DispatchQueue.main.async {
                        self.decrNetActivity()
                        self.title = "Error 1"
                        }
                }
            }
            task.resume()
            return
        }
        
        self.decrNetActivity()
        self.title = "Error 2"
    }
    

    /**
    *  Dado un NSData con el JSON devuelto por el API de busquedas de Twitter, extrae el JSON, y
    *  crea objetos Tweet que añade al array self.tweets.
    *
    *  El JSON es un diccionario con dos claves: "statuses" y "search_metadata".
    *
    *  El valor asociado a la clave "statuses" es un array con los tweets encontrados.
    *
    *  Cada tweet es un diccionario. En esta aplicacion solo cogemos los valores asociados a las claves
    *  "text" y "user". El valor de "text" es el mensaje de tweet. El Valor de "user" es otro diccionario
    *  del que tomamos los valores asociados a las claves "name" y "profile_image_url", que son el nombre
    *  de autor y la url a su foto.
    *
    *  El valor asociado a la clave "search_metadata" es diccionaro del que solo nos interesa el valor
    *  asociado a la clave "next_results". Es la query a utilizar para bajar la siguiente pagina de tweets.
    *
    *  El parametro jsonData es un NSData conteniendo el JSON devuelto por twitter.
    */
    private func addJSONData(jsonData: Data) {
        
        do {
            if let dic = try JSONSerialization.jsonObject(with: jsonData) as? [String:AnyObject] {
                
                // println(dic)
                
                // Extrae los tweets y los almacena en self.tweets.
                // Y actualiza la Table View.
                for tweetDic in dic["statuses"] as! [[String:AnyObject]] {
                    let text  = tweetDic["text"] as! String
                    let username = (tweetDic["user"] as! [String:AnyObject])["name"] as! String
                    let imageUrl = (tweetDic["user"] as! [String:AnyObject])["profile_image_url"] as! String
                    
                    let tweet = Tweet(text: text, username: username, imageUrl: imageUrl)
                    
                    DispatchQueue.main.async {
                        self.tweets += [tweet]
                        let ip = IndexPath(row: self.tweets.count-1, section:0)
                        self.tableView.insertRows(at: [ip], with: .automatic)
                    }
                }
                
                DispatchQueue.main.async {
                    // La query a utilizar para bajar la siguiente pagina de tweets.
                    self.current_query = nil
                    if let nextQuery = (dic["search_metadata"] as! [String:AnyObject])["next_results"] as? String {
                        self.downloadButtonEnabled = true
                        self.current_query = nextQuery.addingPercentEncoding(withAllowedCharacters: .urlQueryAllowed)
                    }
                    
                    self.decrNetActivity()
                    self.title = self.query
                }
            } else {
                
                print("Error: El JSON no es el diccionario esperado.")
                
                DispatchQueue.main.async {
                    self.decrNetActivity()
                    self.title = "Error Formato"
                }
            }
        } catch let err {
            
            print("Error parsing JSON = \(err)")
            
            DispatchQueue.main.async {
                self.decrNetActivity()
                self.title = "Error JSON"
            }
        }
    }
    
    // MARK: - Network Activity
    
    
    /**
    *  Contador usado para saber cuantas veces hemos ejecutado:
    *    UIApplication.sharedApplication().networkActivityIndicatorVisible = true
    *  Esta sentencia activa el indicador de actividad en la red.
    *  Mientras este contador sea mauor que cero no pararemos el indicador.
    */
    private var netActivityCredit = 0

    /**
    *  Activa el indicador de actividad de red.
    */
    private func incrNetActivity() {
        netActivityCredit += 1
        UIApplication.shared.isNetworkActivityIndicatorVisible = true
    }
    
    /**
    *  Desactiva el indicador de actividad de red.
    */
    private func decrNetActivity() {
        netActivityCredit -= 1
        if netActivityCredit < 1 {
            UIApplication.shared.isNetworkActivityIndicatorVisible = false
        }
    }
    
    
    // MARK: - Eventos: acccion del boton para bajar mas tweets
    
    @IBAction func nextPage() {
        
        downloadButtonEnabled = false

        downloadNextPage()
    }
    
 

    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 2
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        switch section {
        case 0: // Seccion con los tweets.
            return tweets.count
        default: // Secccion con el boton de bajar mas tweets.
            return 1
        }
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        switch indexPath.section {
        case 0:
            let cell = tableView.dequeueReusableCell(withIdentifier: "Tweet Cell", for: indexPath) as! TweetCell
            
            cell.msgLabel.text = tweets[indexPath.row].text
            cell.usernameLabel.text = tweets[indexPath.row].username
            
            let url = tweets[indexPath.row].imageUrl
            
            if let image = images[url] {
                cell.userImageView.image = image
            } else {
                cell.userImageView.image = nil
                
                if let url = URL(string: tweets[indexPath.row].imageUrl) {
                    
                    incrNetActivity()
                    
                    let task = session.dataTask(with: url) { (data: Data?, res: URLResponse?, error: Error?) in
                        if error == nil && (res as! HTTPURLResponse).statusCode == 200 {
                            // La imagen descargada.
                            if let newImg = UIImage(data: data!) {
                                DispatchQueue.main.async(execute: {
                                    // Guardar la imagen
                                    self.images[self.tweets[indexPath.row].imageUrl] = newImg
                                    
                                    // Poner la nueva imagen en la fila de la tabla y actualizar la tabla
                                    if let cell = self.tableView.cellForRow(at: indexPath) as? TweetCell {
                                        cell.userImageView.image = newImg;
                                    }
                                })
                            }
                        }
                        DispatchQueue.main.async {
                            self.decrNetActivity()
                        }
                    }
                   task.resume()
                }
            }
            return cell
        default:
            let cell = tableView.dequeueReusableCell(withIdentifier: "Next Page Cell", for: indexPath) as! NextPageCell
            
            cell.downloadButton.isEnabled = downloadButtonEnabled

            return cell
        }
    }

}
